@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -29,7 +29,7 @@
 #endif
 #endif
 
-#if !defined(HAVE_NO_POST)
+#if defined(HAVE_POST)
 #include "PView.h"
 #endif
 
@@ -48,44 +48,47 @@ const char *GetGmshBuildOptions(){ return GMSH_CONFIG_OPTIONS; }
 void PrintUsage(const char *name)
 {
   // If you make changes in this routine, please also change the
-  // texinfo documentation (doc/texinfo/command_line.texi) and the man
-  // page (doc/gmsh.1)
+  // texinfo documentation (doc/texinfo/gmsh.texi) and the man page
+  // (doc/gmsh.1)
   Msg::Direct("Usage: %s [options] [files]", name);
   Msg::Direct("Geometry options:");
   Msg::Direct("  -0                    Output unrolled geometry, then exit");
   Msg::Direct("  -tol float            Set geometrical tolerance");
-  Msg::Direct("  -match int            Match geometries and meshes if int != 0");
+  Msg::Direct("  -match                Match geometries and meshes");
   Msg::Direct("Mesh options:");
   Msg::Direct("  -1, -2, -3            Perform 1D, 2D or 3D mesh generation, then exit");
   Msg::Direct("  -refine               Perform uniform mesh refinement, then exit");
   Msg::Direct("  -part int             Partition after batch mesh generation");
+  Msg::Direct("  -renumber             Renumber the mesh elements after batch mesh generation");
   Msg::Direct("  -saveall              Save all elements (discard physical group definitions)");
-  Msg::Direct("  -o file               Specify mesh output file name");
-  Msg::Direct("  -format string        Set output mesh format (msh, msh1, msh2, unv, vrml, stl, mesh,");
-  Msg::Direct("                          bdf, p3d, cgns, med)");
+  Msg::Direct("  -o file               Specify output file name");
   Msg::Direct("  -bin                  Use binary format when available");  
   Msg::Direct("  -parametric           Save vertices with their parametric coordinates");  
   Msg::Direct("  -numsubedges          Set the number of subdivisions when displaying high order elements");  
   Msg::Direct("  -algo string          Select mesh algorithm (meshadapt, del2d, front2d, del3d, front3d)");
   Msg::Direct("  -smooth int           Set number of mesh smoothing steps");
-  Msg::Direct("  -optimize[_netgen]    Optimize quality of tetrahedral elements");
   Msg::Direct("  -order int            Set mesh order (1, ..., 5)");
+  Msg::Direct("  -optimize[_netgen]    Optimize quality of tetrahedral elements");
   Msg::Direct("  -optimize_hom         Optimize higher order meshes (in 2D)");
-  Msg::Direct("  -clscale float        Set characteristic length scaling factor");
-  Msg::Direct("  -clmin float          Set minimum characteristic length");
-  Msg::Direct("  -clmax float          Set maximum characteristic length");
-  Msg::Direct("  -clcurv               Compute characteristic lengths from curvatures");
+  Msg::Direct("  -optimize_lloyd       Optimize 2D meshes using Lloyd algorithm");
+  Msg::Direct("  -clscale float        Set global mesh element size scaling factor");
+  Msg::Direct("  -clmin float          Set minimum mesh element size");
+  Msg::Direct("  -clmax float          Set maximum mesh element size");
+  Msg::Direct("  -anisoMax float       Set maximum anisotropy (only used in bamg for now)");
+  Msg::Direct("  -smoothRatio float    Set smoothing ration between mesh sizes at nodes of a same edge (only used in bamg)");
+  Msg::Direct("  -clcurv               Automatically compute element sizes from curvatures");
   Msg::Direct("  -epslc1d              Set the accuracy of the evaluation of the LCFIELD for 1D mesh");
   Msg::Direct("  -swapangle            Set the treshold angle (in degree) between two adjacent faces");
   Msg::Direct("                          below which a swap is allowed");
   Msg::Direct("  -rand float           Set random perturbation factor");
   Msg::Direct("  -bgm file             Load background mesh from file");
+  Msg::Direct("  -check                Perform various consistency checks on mesh");
 #if defined(HAVE_FLTK)
   Msg::Direct("Post-processing options:");
-  Msg::Direct("  -noview               Hide all views on startup");
   Msg::Direct("  -link int             Select link mode between views (0, 1, 2, 3, 4)");
   Msg::Direct("  -combine              Combine views having identical names into multi-time-step views");
   Msg::Direct("Display options:");    
+  Msg::Direct("  -n                    Hide all meshes and post-processing views on startup");
   Msg::Direct("  -nodb                 Disable double buffering");
   Msg::Direct("  -fontsize int         Specify the font size for the GUI");
   Msg::Direct("  -theme string         Specify FLTK GUI theme");
@@ -95,9 +98,13 @@ void PrintUsage(const char *name)
   Msg::Direct("  -                     Parse input files, then exit");
 #if defined(HAVE_FLTK)
   Msg::Direct("  -a, -g, -m, -s, -p    Start in automatic, geometry, mesh, solver or post-processing mode");
+#endif
+#if defined(HAVE_LUA)
+  Msg::Direct("  -lua                  Start an interactive lua session");
 #endif
   Msg::Direct("  -pid                  Print process id on stdout");
   Msg::Direct("  -listen               Always listen to incoming connection requests");
+  Msg::Direct("  -watch pattern        Pattern of files to merge as they become available");
   Msg::Direct("  -v int                Set verbosity level");
   Msg::Direct("  -nopopup              Don't popup dialog windows in scripts");
   Msg::Direct("  -string \"string\"      Parse option string at startup");
@@ -114,7 +121,7 @@ void GetOptions(int argc, char *argv[])
   int terminal = CTX::instance()->terminal;
   CTX::instance()->terminal = 1;
 
-#if !defined(HAVE_NO_PARSER)
+#if defined(HAVE_PARSER)
   // Parse session and option files
   ParseFile(CTX::instance()->homeDir + CTX::instance()->sessionFileName, true);
   ParseFile(CTX::instance()->homeDir + CTX::instance()->optionsFileName, true);
@@ -126,15 +133,19 @@ void GetOptions(int argc, char *argv[])
 
     if(argv[i][0] == '-') {
 
-      if(!strcmp(argv[i] + 1, "socket")) {
+      if(!strcmp(argv[i] + 1, "")) {
+        CTX::instance()->batch = -99;
+        i++;
+      }
+      else if(!strcmp(argv[i] + 1, "socket")) {
         i++;        
         if(argv[i])
-          CTX::instance()->solver.socketName = argv[i++];
+          Msg::InitClient(argv[i++]);
         else
           Msg::Fatal("Missing string");
         CTX::instance()->batch = -3;
       }
-      else if(!strcmp(argv[i] + 1, "")) {
+      else if(!strcmp(argv[i] + 1, "check")) {
         CTX::instance()->batch = -2;
         i++;
       }
@@ -162,10 +173,15 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->batch = 5;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "renumber")) {
+        CTX::instance()->batchAfterMesh = 1;
+        CTX::instance()->partitionOptions.renumber = 1;
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "part")) {
         i++;
         if(argv[i]){
-          CTX::instance()->batchAfterMesh = 1;
+          CTX::instance()->batchAfterMesh = 1 ;
           opt_mesh_partition_num(0, GMSH_SET, atoi(argv[i++]));
         }
         else
@@ -212,10 +228,30 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->mesh.optimizeNetgen = 1;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "optimize_hom")) {
+        i++;
+        opt_mesh_smooth_internal_edges(0, GMSH_SET, 1);
+      }
+      else if(!strcmp(argv[i] + 1, "optimize_lloyd")) {
+        i++;
+        CTX::instance()->mesh.optimizeLloyd = 1;
+      }
       else if(!strcmp(argv[i] + 1, "nopopup")) {
         CTX::instance()->noPopup = 1;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "watch")) {
+        i++;
+        if(argv[i]){
+          std::string tmp = argv[i++];
+          if(tmp.size() > 2 && tmp[0] == '"' && tmp[tmp.size() - 1] == '"')
+            CTX::instance()->watchFilePattern = tmp.substr(1, tmp.size() - 2);
+          else
+            CTX::instance()->watchFilePattern = tmp;
+        }
+        else
+          Msg::Fatal("Missing string");
+      }
       else if(!strcmp(argv[i] + 1, "string")) {
         i++;
         if(argv[i])
@@ -237,6 +273,20 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing file name");
       }
+      else if(!strcmp(argv[i] + 1, "anisoMax")) {
+        i++;
+        if(argv[i])
+          CTX::instance()->mesh.anisoMax = atof(argv[i++]);
+        else
+          Msg::Fatal("Missing anisotropy ratio");
+      }
+      else if(!strcmp(argv[i] + 1, "smoothRatio")) {
+        i++;
+        if(argv[i])
+          CTX::instance()->mesh.smoothRatio = atof(argv[i++]);
+        else
+          Msg::Fatal("Missing smooth ratio");
+      }
       else if(!strcmp(argv[i] + 1, "bgm")) {
         i++;
         if(argv[i])
@@ -263,11 +313,11 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->batch = 1;
         while(i < argc) {
           std::string fileName = std::string(argv[i]) + "_new";
-#if !defined(HAVE_NO_POST)
+#if defined(HAVE_POST)
           unsigned int n = PView::list.size();
 #endif
           OpenProject(argv[i]);
-#if !defined(HAVE_NO_POST)
+#if defined(HAVE_POST)
           // convert post-processing views to latest binary format
           for(unsigned int j = n; j < PView::list.size(); j++)
             PView::list[j]->write(fileName, 1, (j == n) ? false : true);
@@ -291,10 +341,7 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "match")) {
         i++;
-        if(argv[1])
-          CTX::instance()->geom.matchGeomAndMesh = atoi(argv[i++]);
-        else
-          Msg::Fatal("Missing number");
+        CTX::instance()->geom.matchGeomAndMesh = 1;
       }
       else if(!strcmp(argv[i] + 1, "scale")) {
         i++;
@@ -322,7 +369,7 @@ void GetOptions(int argc, char *argv[])
         if(argv[i]) {
           CTX::instance()->mesh.lcFactor = atof(argv[i++]);
           if(CTX::instance()->mesh.lcFactor <= 0.0)
-            Msg::Fatal("Characteristic length factor must be > 0");
+            Msg::Fatal("Mesh element size factor must be > 0");
         }
         else
           Msg::Fatal("Missing number");
@@ -419,48 +466,6 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing argument");
       }
-      else if(!strcmp(argv[i] + 1, "optimize_hom")) {
-        i++;
-        opt_mesh_smooth_internal_edges(0, GMSH_SET, 1);
-      }
-      else if(!strcmp(argv[i] + 1, "format") || !strcmp(argv[i] + 1, "f")) {
-        i++;
-        if(argv[i]) {
-          if(!strcmp(argv[i], "msh1")){
-            CTX::instance()->mesh.format = FORMAT_MSH;
-            CTX::instance()->mesh.mshFileVersion = 1.0;
-          }
-          else if(!strcmp(argv[i], "msh2")){
-            CTX::instance()->mesh.format = FORMAT_MSH;
-            CTX::instance()->mesh.mshFileVersion = 2.0;
-          }
-          else if(!strcmp(argv[i], "msh"))
-            CTX::instance()->mesh.format = FORMAT_MSH;
-          else if(!strcmp(argv[i], "unv"))
-            CTX::instance()->mesh.format = FORMAT_UNV;
-          else if(!strcmp(argv[i], "vrml"))
-            CTX::instance()->mesh.format = FORMAT_VRML;
-          else if(!strcmp(argv[i], "stl"))
-            CTX::instance()->mesh.format = FORMAT_STL;
-          else if(!strcmp(argv[i], "mesh"))
-            CTX::instance()->mesh.format = FORMAT_MESH;
-          else if(!strcmp(argv[i], "bdf"))
-            CTX::instance()->mesh.format = FORMAT_BDF;
-          else if(!strcmp(argv[i], "p3d"))
-            CTX::instance()->mesh.format = FORMAT_P3D;
-          else if(!strcmp(argv[i], "cgns"))
-            CTX::instance()->mesh.format = FORMAT_CGNS;
-          else if(!strcmp(argv[i], "diff"))
-            CTX::instance()->mesh.format = FORMAT_DIFF;
-          else if(!strcmp(argv[i], "med"))
-            CTX::instance()->mesh.format = FORMAT_MED;
-          else
-            Msg::Fatal("Unknown mesh format");
-          i++;
-        }
-        else
-          Msg::Fatal("Missing format");
-      }
       else if(!strcmp(argv[i] + 1, "bin")) {
         i++;
         CTX::instance()->mesh.binary = 1;
@@ -480,6 +485,8 @@ void GetOptions(int argc, char *argv[])
             CTX::instance()->mesh.algo2d = ALGO_2D_DELAUNAY;
           else if(!strncmp(argv[i], "front2d", 7) || !strncmp(argv[i], "frontal", 7))
             CTX::instance()->mesh.algo2d = ALGO_2D_FRONTAL;
+          else if(!strncmp(argv[i], "bamg",4))
+            CTX::instance()->mesh.algo2d = ALGO_2D_BAMG;
           else if(!strncmp(argv[i], "del3d", 5) || !strncmp(argv[i], "tetgen", 6))
             CTX::instance()->mesh.algo2d = ALGO_3D_DELAUNAY;
           else if(!strncmp(argv[i], "front3d", 7) || !strncmp(argv[i], "netgen", 6))
@@ -491,6 +498,51 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing algorithm");
       }
+      else if(!strcmp(argv[i] + 1, "format") || !strcmp(argv[i] + 1, "f")) {
+        i++;
+        if(argv[i]) {
+          if(!strcmp(argv[i], "auto")){
+            CTX::instance()->mesh.fileFormat = FORMAT_AUTO;
+          }
+          else if(!strcmp(argv[i], "msh1")){
+            CTX::instance()->mesh.fileFormat = FORMAT_MSH;
+            CTX::instance()->mesh.mshFileVersion = 1.0;
+          }
+          else if(!strcmp(argv[i], "msh2")){
+            CTX::instance()->mesh.fileFormat = FORMAT_MSH;
+            CTX::instance()->mesh.mshFileVersion = 2.0;
+          }
+          else if(!strcmp(argv[i], "msh"))
+            CTX::instance()->mesh.fileFormat = FORMAT_MSH;
+          else if(!strcmp(argv[i], "unv"))
+            CTX::instance()->mesh.fileFormat = FORMAT_UNV;
+          else if(!strcmp(argv[i], "vrml"))
+            CTX::instance()->mesh.fileFormat = FORMAT_VRML;
+	  else if(!strcmp(argv[i], "ply2"))
+	    CTX::instance()->mesh.fileFormat = FORMAT_PLY2;
+          else if(!strcmp(argv[i], "stl"))
+            CTX::instance()->mesh.fileFormat = FORMAT_STL;
+          else if(!strcmp(argv[i], "mesh"))
+            CTX::instance()->mesh.fileFormat = FORMAT_MESH;
+          else if(!strcmp(argv[i], "bdf"))
+            CTX::instance()->mesh.fileFormat = FORMAT_BDF;
+          else if(!strcmp(argv[i], "p3d"))
+            CTX::instance()->mesh.fileFormat = FORMAT_P3D;
+          else if(!strcmp(argv[i], "cgns"))
+            CTX::instance()->mesh.fileFormat = FORMAT_CGNS;
+          else if(!strcmp(argv[i], "diff"))
+            CTX::instance()->mesh.fileFormat = FORMAT_DIFF;
+          else if(!strcmp(argv[i], "med"))
+            CTX::instance()->mesh.fileFormat = FORMAT_MED;
+          else if(!strcmp(argv[i], "ir3"))
+            CTX::instance()->mesh.fileFormat = FORMAT_IR3;
+          else
+            Msg::Fatal("Unknown mesh format");
+          i++;
+        }
+        else
+          Msg::Fatal("Missing format");
+      }
       else if(!strcmp(argv[i] + 1, "listen")) {
         CTX::instance()->solver.listen = 1;
         i++;
@@ -519,7 +571,7 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "help") || !strcmp(argv[i] + 1, "-help")) {
         fprintf(stderr, "Gmsh, a 3D mesh generator with pre- and post-processing facilities\n");
-        fprintf(stderr, "Copyright (C) 1997-2009 Christophe Geuzaine and Jean-Francois Remacle\n");
+        fprintf(stderr, "Copyright (C) 1997-2010 Christophe Geuzaine and Jean-Francois Remacle\n");
         PrintUsage(argv[0]);
         Msg::Exit(0);
       }
@@ -547,6 +599,25 @@ void GetOptions(int argc, char *argv[])
         opt_view_visible(0, GMSH_SET, 0);
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "nomesh")) {
+        opt_mesh_points(0, GMSH_SET, 0.);
+        opt_mesh_lines(0, GMSH_SET, 0.);
+        opt_mesh_surfaces_edges(0, GMSH_SET, 0.);
+        opt_mesh_surfaces_faces(0, GMSH_SET, 0.);
+        opt_mesh_volumes_edges(0, GMSH_SET, 0.);
+        opt_mesh_volumes_faces(0, GMSH_SET, 0.);
+        i++;
+      }
+      else if(!strcmp(argv[i] + 1, "n")) {
+        opt_view_visible(0, GMSH_SET, 0);
+        opt_mesh_points(0, GMSH_SET, 0.);
+        opt_mesh_lines(0, GMSH_SET, 0.);
+        opt_mesh_surfaces_edges(0, GMSH_SET, 0.);
+        opt_mesh_surfaces_faces(0, GMSH_SET, 0.);
+        opt_mesh_volumes_edges(0, GMSH_SET, 0.);
+        opt_mesh_volumes_faces(0, GMSH_SET, 0.);
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "link")) {
         i++;
         if(argv[i])
@@ -595,6 +666,12 @@ void GetOptions(int argc, char *argv[])
           Msg::Fatal("Missing argument");
       }
 #endif
+#if defined (HAVE_LUA)
+      else if(!strcmp(argv[i] + 1, "lua")) {
+        i++;
+        CTX::instance()->batch = -4;
+      }
+#endif
 #if defined(__APPLE__)
       else if(!strncmp(argv[i] + 1, "psn", 3)) {
         // The Mac Finder launches programs with a special command
