@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,10 +8,12 @@
 
 #include <string>
 
+const char *GetEnvironmentVar(const char *var);
 double GetTimeInSeconds();
 void SleepInSeconds(double s);
 void CheckResources();
 double Cpu();
+long GetMemoryUsage();
 int GetProcessId();
 std::string GetHostName();
 int UnlinkFile(std::string fileName);
