@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -39,10 +39,17 @@ std::string opt_general_tmp_filename(OPT_ARGS_STR);
 std::string opt_general_error_filename(OPT_ARGS_STR);
 std::string opt_general_session_filename(OPT_ARGS_STR);
 std::string opt_general_options_filename(OPT_ARGS_STR);
+std::string opt_general_recent_file1(OPT_ARGS_STR);
+std::string opt_general_recent_file2(OPT_ARGS_STR);
+std::string opt_general_recent_file3(OPT_ARGS_STR);
+std::string opt_general_recent_file4(OPT_ARGS_STR);
+std::string opt_general_recent_file5(OPT_ARGS_STR);
 std::string opt_general_editor(OPT_ARGS_STR);
 std::string opt_general_web_browser(OPT_ARGS_STR);
+std::string opt_general_watch_file_pattern(OPT_ARGS_STR);
 std::string opt_general_gui_theme(OPT_ARGS_STR);
 std::string opt_general_graphics_font(OPT_ARGS_STR);
+std::string opt_general_graphics_font_title(OPT_ARGS_STR);
 std::string opt_solver_socket_name(OPT_ARGS_STR);
 std::string opt_solver_name(OPT_ARGS_STR);
 std::string opt_solver_name0(OPT_ARGS_STR);
@@ -56,6 +63,12 @@ std::string opt_solver_executable1(OPT_ARGS_STR);
 std::string opt_solver_executable2(OPT_ARGS_STR);
 std::string opt_solver_executable3(OPT_ARGS_STR);
 std::string opt_solver_executable4(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments0(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments1(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments2(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments3(OPT_ARGS_STR);
+std::string opt_solver_extra_arguments4(OPT_ARGS_STR);
 std::string opt_solver_help(OPT_ARGS_STR);
 std::string opt_solver_help0(OPT_ARGS_STR);
 std::string opt_solver_help1(OPT_ARGS_STR);
@@ -222,6 +235,7 @@ std::string opt_view_stipple9(OPT_ARGS_STR);
 double opt_general_initial_context(OPT_ARGS_NUM);
 double opt_general_fontsize(OPT_ARGS_NUM);
 double opt_general_graphics_fontsize(OPT_ARGS_NUM);
+double opt_general_graphics_fontsize_title(OPT_ARGS_NUM);
 double opt_general_graphics_position0(OPT_ARGS_NUM);
 double opt_general_graphics_position1(OPT_ARGS_NUM);
 double opt_general_graphics_size0(OPT_ARGS_NUM);
@@ -325,6 +339,7 @@ double opt_general_small_axes_size(OPT_ARGS_NUM);
 double opt_general_quadric_subdivisions(OPT_ARGS_NUM);
 double opt_general_double_buffer(OPT_ARGS_NUM);
 double opt_general_antialiasing(OPT_ARGS_NUM);
+double opt_general_stereo(OPT_ARGS_NUM);
 double opt_general_alpha_blending(OPT_ARGS_NUM);
 double opt_general_vector_type(OPT_ARGS_NUM);
 double opt_general_arrow_head_radius(OPT_ARGS_NUM);
@@ -443,7 +458,7 @@ double opt_geometry_clip(OPT_ARGS_NUM);
 double opt_geometry_copy_meshing_method(OPT_ARGS_NUM);
 double opt_geometry_exact_extrusion(OPT_ARGS_NUM);
 double opt_geometry_match_geom_and_mesh(OPT_ARGS_NUM);
-double opt_mesh_label_frequency(OPT_ARGS_NUM);
+double opt_mesh_label_sampling(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
 double opt_mesh_remove_4_triangles(OPT_ARGS_NUM);
@@ -489,17 +504,30 @@ double opt_mesh_point_type(OPT_ARGS_NUM);
 double opt_mesh_line_width(OPT_ARGS_NUM);
 double opt_mesh_reverse_all_normals(OPT_ARGS_NUM);
 double opt_mesh_smooth_normals(OPT_ARGS_NUM);
+double opt_mesh_smooth_ratio(OPT_ARGS_NUM);
 double opt_mesh_angle_smooth_normals(OPT_ARGS_NUM);
+double opt_mesh_aniso_max(OPT_ARGS_NUM);
 double opt_mesh_light(OPT_ARGS_NUM);
 double opt_mesh_light_lines(OPT_ARGS_NUM);
 double opt_mesh_light_two_side(OPT_ARGS_NUM);
-double opt_mesh_format(OPT_ARGS_NUM);
+double opt_mesh_file_format(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
+double opt_mesh_msh_file_partitioned(OPT_ARGS_NUM);
+double opt_mesh_partition_hex_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_pri_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_pyr_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_qua_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_tet_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_tri_weight(OPT_ARGS_NUM);
 double opt_mesh_binary(OPT_ARGS_NUM);
 double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
+double opt_mesh_algo_recombine(OPT_ARGS_NUM);
+double opt_mesh_recombine_all(OPT_ARGS_NUM);
+double opt_mesh_remesh_algo(OPT_ARGS_NUM);
+double opt_mesh_remesh_param(OPT_ARGS_NUM);
 double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
 double opt_mesh_mesh_only_visible(OPT_ARGS_NUM);
 double opt_mesh_min_circ_points(OPT_ARGS_NUM);
@@ -548,6 +576,7 @@ double opt_mesh_partition_chaco_terminal_propogation(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_algorithm(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_refine_algorithm(OPT_ARGS_NUM);
+double opt_mesh_partition_by_extrusion(OPT_ARGS_NUM);
 double opt_mesh_clip(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
@@ -649,6 +678,7 @@ double opt_view_axes_zmax(OPT_ARGS_NUM);
 double opt_view_position0(OPT_ARGS_NUM);
 double opt_view_position1(OPT_ARGS_NUM);
 double opt_view_auto_position(OPT_ARGS_NUM);
+double opt_view_sampling(OPT_ARGS_NUM);
 double opt_view_size0(OPT_ARGS_NUM);
 double opt_view_size1(OPT_ARGS_NUM);
 double opt_view_nb_iso(OPT_ARGS_NUM);
@@ -685,7 +715,17 @@ double opt_view_line_width(OPT_ARGS_NUM);
 double opt_view_point_type(OPT_ARGS_NUM);
 double opt_view_line_type(OPT_ARGS_NUM);
 double opt_view_clip(OPT_ARGS_NUM);
-double opt_print_format(OPT_ARGS_NUM);
+double opt_view_force_num_components(OPT_ARGS_NUM);
+double opt_view_component_map0(OPT_ARGS_NUM);
+double opt_view_component_map1(OPT_ARGS_NUM);
+double opt_view_component_map2(OPT_ARGS_NUM);
+double opt_view_component_map3(OPT_ARGS_NUM);
+double opt_view_component_map4(OPT_ARGS_NUM);
+double opt_view_component_map5(OPT_ARGS_NUM);
+double opt_view_component_map6(OPT_ARGS_NUM);
+double opt_view_component_map7(OPT_ARGS_NUM);
+double opt_view_component_map8(OPT_ARGS_NUM);
+double opt_print_file_format(OPT_ARGS_NUM);
 double opt_print_eps_compress(OPT_ARGS_NUM);
 double opt_print_eps_ps3shading(OPT_ARGS_NUM);
 double opt_print_eps_quality(OPT_ARGS_NUM);
@@ -710,6 +750,7 @@ double opt_print_gif_transparent(OPT_ARGS_NUM);
 double opt_print_text(OPT_ARGS_NUM);
 double opt_print_tex_as_equation(OPT_ARGS_NUM);
 double opt_print_composite_windows(OPT_ARGS_NUM);
+double opt_print_delete_tmp_files(OPT_ARGS_NUM);
 
 // COLORS
 
