@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -62,7 +62,7 @@ std::string SanitizeTeXString(const char *in, int equation)
   return out;
 }
 
-std::string FixWindowsPath(std::string in)
+std::string FixWindowsPath(const std::string &in)
 {
 #if defined(__CYGWIN__)
   char tmp[1024];
@@ -73,7 +73,7 @@ std::string FixWindowsPath(std::string in)
 #endif
 }
 
-std::string FixRelativePath(std::string reference, std::string in)
+std::string FixRelativePath(const std::string &reference, const std::string &in)
 {
   if(in.empty()) return "";
 
@@ -88,7 +88,7 @@ std::string FixRelativePath(std::string reference, std::string in)
   }
 }
 
-std::vector<std::string> SplitFileName(std::string fileName)
+std::vector<std::string> SplitFileName(const std::string &fileName)
 {
   // returns [path, baseName, extension]
   int idot = fileName.find_last_of('.');
@@ -104,28 +104,45 @@ std::vector<std::string> SplitFileName(std::string fileName)
   return s;
 }
 
-std::vector<std::string> SplitWhiteSpace(std::string in, unsigned int len)
+std::string GetFileNameWithoutPath(const std::string &fileName)
 {
-  std::vector<std::string> out(1);
-  for(unsigned int i = 0; i < in.size(); i++){
-    out.back() += in[i];
-    if(out.back().size() > len && in[i] == ' ')
-      out.resize(out.size() + 1);
-  }
+  std::vector<std::string> s = SplitFileName(fileName);
+  return s[1] + s[2];
+}
+
+std::string ConvertFileToString(const std::string &fileName)
+{
+  FILE *fp = fopen(fileName.c_str(), "r");
+  if(!fp) return "";
+  std::string out;
+  char str[256];
+  while(!feof(fp) && fgets(str, sizeof(str), fp)) out += str;
+  fclose(fp);
   return out;
 }
 
-std::string ReplacePercentS(std::string in, std::string val)
+void ReplaceSubStringInPlace(const std::string &olds, const std::string &news, 
+                             std::string &str)
 {
-  std::string out;
-  for(unsigned int i = 0; i < in.size(); i++){
-    if(in[i] == '%' && i + 1 < in.size() && in[i + 1] == 's'){
-      out += val;
-      i++;
-    }
-    else{
-      out += in[i];
-    }
+  while(1){
+    int pos = str.find(olds.c_str());
+    if(pos == (int)std::string::npos) break;
+    str.replace(pos, olds.size(), news.c_str());
   }
-  return out;
+}
+
+std::string ReplaceSubString(const std::string &olds, const std::string &news, 
+                             const std::string &str)
+{
+  std::string copy(str);
+  ReplaceSubStringInPlace(olds, news, copy);
+  return copy;
+}
+
+void ConvertToHTML(std::string &str)
+{
+  ReplaceSubStringInPlace("<", "&lt;", str);
+  ReplaceSubStringInPlace(">", "&gt;", str);
+  ReplaceSubStringInPlace("\n\n", "<p>", str);
+  ReplaceSubStringInPlace("\n", "<br>", str);
 }
