@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include <string.h>
 #include <FL/fl_draw.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Scroll.H>
@@ -417,10 +418,10 @@ static void proj_hide_cb(Fl_Widget *w, void *data)
 static void save_selection_cb(Fl_Widget *w, void *data)
 {
   projectionEditor *e = (projectionEditor*)data;
-  if(file_chooser(0, 1, "Save Selection", "*.{geo,msh}")){
-    FILE *fp = fopen(file_chooser_get_name(1).c_str(), "w");
+  if(fileChooser(FILE_CHOOSER_CREATE, "Save Selection", "*.{geo,msh}")){
+    FILE *fp = fopen(fileChooserGetName(1).c_str(), "w");
     if(!fp){
-      Msg::Error("Unable to open file `%s'", file_chooser_get_name(1).c_str());
+      Msg::Error("Unable to open file `%s'", fileChooserGetName(1).c_str());
       return;
     }
     std::vector<GEntity*> &ent(e->getEntities());
@@ -457,10 +458,10 @@ static void save_selection_cb(Fl_Widget *w, void *data)
 static void load_projection_cb(Fl_Widget *w, void *data)
 {
   projectionEditor *e = (projectionEditor*)data;
-  if(file_chooser(0, 0, "Load Projection", "*.pro")){
-    FILE *fp = fopen(file_chooser_get_name(1).c_str(), "r");
+  if(fileChooser(FILE_CHOOSER_SINGLE, "Load Projection", "*.pro")){
+    FILE *fp = fopen(fileChooserGetName(1).c_str(), "r");
     if(!fp){
-      Msg::Error("Unable to open file `%s'", file_chooser_get_name(1).c_str());
+      Msg::Error("Unable to open file `%s'", fileChooserGetName(1).c_str());
       return;
     }
     int num;
@@ -501,8 +502,8 @@ static void save_projection_cb(Fl_Widget *w, void *data)
   projection *p = e->getCurrentProjection();
   if(p){
     FM::ProjectionSurface *ps = (FM::ProjectionSurface*)p->face->getNativePtr();
-    if(file_chooser(0, 1, "Save Projection", "*.pro")){
-      std::string name = file_chooser_get_name(1);
+    if(fileChooser(FILE_CHOOSER_CREATE, "Save Projection", "*.pro")){
+      std::string name = fileChooserGetName(1);
       FILE *fp = fopen(name.c_str(), "w");
       if(!fp){
         Msg::Error("Unable to open file `%s'", name.c_str());
@@ -687,10 +688,10 @@ static void action_cb(Fl_Widget *w, void *data)
       delete_fourier(faces[i]);
   }
   else{
-    if(file_chooser(0, 1, "Save Fourier Model", "*.fm")){
-      FILE *fp = fopen(file_chooser_get_name(1).c_str(), "w");
+    if(fileChooser(FILE_CHOOSER_CREATE, "Save Fourier Model", "*.fm")){
+      FILE *fp = fopen(fileChooserGetName(1).c_str(), "w");
       if(!fp){
-        Msg::Error("Unable to open file `%s'", file_chooser_get_name(1).c_str());
+        Msg::Error("Unable to open file `%s'", fileChooserGetName(1).c_str());
         return;
       }
       fprintf(fp, "%d\n", (int)faces.size());
