@@ -1,4 +1,4 @@
- // Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _SOLVER_WINDOW_H_
 #define _SOLVER_WINDOW_H_
 
+#include <vector>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Input.H>
 #include <FL/Fl_Choice.H>
@@ -15,10 +16,10 @@
 class solverWindow{
  public:
   Fl_Window *win;
-  Fl_Input *input[50];
-  Fl_Choice *choice[10];
+  Fl_Input *input[5];
   Fl_Menu_Button *menu;
-  Fl_Button *command[10];
+  std::vector<Fl_Choice*> choice;
+  std::vector<Fl_Button*> command;
  public:
   solverWindow(int solverIndex, int deltaFontSize=0);
 };
