@@ -1,14 +1,16 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <sstream>
 #include <algorithm>
+#include "GModel.h"
 #include "GVertex.h"
 #include "GFace.h"
 #include "MPoint.h"
 #include "GmshMessage.h"
+#include "Bindings.h"
 
 GVertex::GVertex(GModel *m, int tag, double ms) : GEntity(m, tag), meshSize(ms) 
 {
@@ -25,6 +27,8 @@ void GVertex::deleteMesh()
   mesh_vertices.clear();
   for(unsigned int i = 0; i < points.size(); i++) delete points[i];
   points.clear();
+  deleteVertexArrays();
+  model()->destroyMeshCaches();
 }
 
 void GVertex::setPosition(GPoint &p)
@@ -56,10 +60,17 @@ std::string GVertex::getAdditionalInfoString()
   return sstream.str();
 }
 
-void GVertex::writeGEO(FILE *fp)
+void GVertex::writeGEO(FILE *fp, const std::string &meshSizeParameter)
 {
-  fprintf(fp, "Point(%d) = {%.16g, %.16g, %.16g, %.16g};\n",
-          tag(), x(), y(), z(), prescribedMeshSizeAtVertex());
+  if(meshSizeParameter.size())
+    fprintf(fp, "Point(%d) = {%.16g, %.16g, %.16g, %s};\n",
+            tag(), x(), y(), z(), meshSizeParameter.c_str());
+  else if(prescribedMeshSizeAtVertex() != MAX_LC)
+    fprintf(fp, "Point(%d) = {%.16g, %.16g, %.16g, %.16g};\n",
+            tag(), x(), y(), z(), prescribedMeshSizeAtVertex());
+  else
+    fprintf(fp, "Point(%d) = {%.16g, %.16g, %.16g};\n",
+            tag(), x(), y(), z());
 }
 
 unsigned int GVertex::getNumMeshElements()
@@ -82,3 +93,17 @@ bool GVertex::isOnSeam(const GFace *gf) const
   }
   return false;
 }
+
+void GVertex::registerBindings(binding *b)
+{
+  classBinding *cb = b->addClass<GVertex>("GVertex");
+  cb->setDescription("A GVertex is a geometrical 0D entity");
+  cb->setParentClass<GEntity>();
+  methodBinding *cm;
+  cm = cb->addMethod("x", (double (GVertex::*)() const) &GVertex::x);
+  cm->setDescription("Return the x-coordinate.");
+  cm = cb->addMethod("y", (double (GVertex::*)() const) &GVertex::y);
+  cm->setDescription("Return the y-coordinate.");
+  cm = cb->addMethod("z", (double (GVertex::*)() const) &GVertex::z);
+  cm->setDescription("Return the z-coordinate.");
+}
