@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -18,5 +18,8 @@ int MPyramid::getVolumeSign()
   mat[2][0] = _v[1]->z() - _v[0]->z();
   mat[2][1] = _v[3]->z() - _v[0]->z();
   mat[2][2] = _v[4]->z() - _v[0]->z();
-  return sign(det3x3(mat));
+  double d = det3x3(mat);
+  if(d < 0.) return -1;
+  else if(d > 0.) return 1;
+  else return 0;
 }
