@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -14,14 +14,18 @@
 class OCCRegion : public GRegion {
  protected:
   TopoDS_Solid s;
+  void replaceFacesInternal (std::list<GFace*> &);
+  void setup();
  public:
-  OCCRegion(GModel *m, TopoDS_Solid s, int num, TopTools_IndexedMapOfShape &fmap);
+  OCCRegion(GModel *m, TopoDS_Solid s, int num);
   virtual ~OCCRegion() {}
   virtual GeomType geomType() const;
   ModelType getNativeType() const { return OpenCascadeModel; }
   void * getNativePtr() const { return (void*)&s; }
+  TopoDS_Solid getTopoDS_Shape() {return s;}
 };
 
+GRegion *getOCCRegionByNativePtr(GModel *model, TopoDS_Solid toFind);
 #endif
 
 #endif
