@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -30,6 +30,13 @@ class SPoint2 {
   void operator*=(double mult);
   SPoint2 operator*(double mult);
   operator double *() { return P; }
+  bool operator < (const SPoint2  &other) const
+  {
+    if(other.P[0] < P[0]) return true;
+    if(other.P[0] > P[0]) return false;
+    if(other.P[1] < P[1]) return true;
+    return false;
+  }
 };
 
 inline SPoint2 operator + (const SPoint2 &a, const SPoint2 &b)
