@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -14,22 +14,25 @@ class discreteEdge : public GEdge {
  protected:
   std::vector<double> _pars;
   std::vector<int> _orientation;
-  std::map<MVertex*,MLine*> boundv;
-  mutable std::map<MVertex*,SVector3> _normals;
+  std::map<MVertex*, MLine*> boundv;
   bool createdTopo;
  public:
   discreteEdge(GModel *model, int num, GVertex *_v0, GVertex *_v1);
   virtual ~discreteEdge() {}
-  void getLocalParameter(const double &t, int &iEdge, double &tLoc) const;
+  bool getLocalParameter(const double &t, int &iEdge, double &tLoc) const;
   virtual GeomType geomType() const { return DiscreteCurve; }
   virtual GPoint point(double p) const;
   virtual SVector3 firstDer(double par) const;
   virtual Range<double> parBounds(int) const;
-  void parametrize();
+  void parametrize(std::map<GFace*, std::map<MVertex*, MVertex*, 
+                   std::less<MVertex*> > > &face2Verts, 
+                   std::map<GRegion*, std::map<MVertex*, MVertex*,
+                   std::less<MVertex*> > > &region2Vert);
   void orderMLines();
   void setBoundVertices();
   void createTopo();
   void computeNormals () const;
+  void writeGEO(FILE *fp);
 };
 
 #endif
