@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -19,10 +19,7 @@ class fourierVertex : public GVertex {
  protected:
   FM::TopoVertex *v;
  public:
-  fourierVertex(GModel *m, int num, FM::TopoVertex* _v) : GVertex(m, num), v(_v)
-  {
-    mesh_vertices.push_back(new MVertex(x(), y(), z(), this));
-  }
+  fourierVertex(GModel *m, int num, FM::TopoVertex* _v) : GVertex(m, num), v(_v){}
   virtual ~fourierVertex() {}
   virtual GPoint point() const { return GPoint(x(),y(),z()); }
   virtual double x() const { return v->GetX(); }
