@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -15,13 +15,15 @@ class gmshFace : public GFace {
   bool isSphere;
   SPoint3 center;
   double radius;
+  bool buildSTLTriangulation(bool force);
  public:
   gmshFace(GModel *m, Surface *face);
   virtual ~gmshFace(){}
   Range<double> parBounds(int i) const; 
   void setModelEdges(std::list<GEdge*> &);
   virtual GPoint point(double par1, double par2) const; 
-  virtual GPoint closestPoint(const SPoint3 &queryPoint, const double initialGuess[2]) const; 
+  virtual GPoint closestPoint(const SPoint3 &queryPoint, 
+                              const double initialGuess[2]) const; 
   virtual bool containsPoint(const SPoint3 &pt) const;  
   virtual double getMetricEigenvalue(const SPoint2 &);  
   virtual SVector3 normal(const SPoint2 &param) const; 
@@ -30,7 +32,7 @@ class gmshFace : public GFace {
   virtual GEntity::GeomType geomType() const; 
   ModelType getNativeType() const { return GmshModel; }
   void *getNativePtr() const { return s; }
-  virtual SPoint2 parFromPoint(const SPoint3 &) const;
+  virtual SPoint2 parFromPoint(const SPoint3 &, bool onSurface=true) const;
   virtual void resetMeshAttributes();
 };
 
