@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -72,7 +72,7 @@ class drawGVertex {
       char Num[100];
       sprintf(Num, "%d", v->tag());
       double offset = (0.5 * CTX::instance()->geom.pointSize + 
-                       0.3 * CTX::instance()->glFontSize) * _ctx->pixel_equiv_x;
+                       0.1 * CTX::instance()->glFontSize) * _ctx->pixel_equiv_x;
       glRasterPos3d(x + offset / _ctx->s[0],
                     y + offset / _ctx->s[1],
                     z + offset / _ctx->s[2]);
@@ -166,7 +166,7 @@ class drawGEdge {
       char Num[100];
       sprintf(Num, "%d", e->tag());
       double offset = (0.5 * CTX::instance()->geom.lineWidth + 
-                       0.3 * CTX::instance()->glFontSize) * _ctx->pixel_equiv_x;
+                       0.1 * CTX::instance()->glFontSize) * _ctx->pixel_equiv_x;
       double x = p.x(), y = p.y(), z = p.z();
       _ctx->transform(x, y, z);
       glRasterPos3d(x + offset / _ctx->s[0],
@@ -236,7 +236,11 @@ class drawGFace {
   }
   void _drawParametricGFace(GFace *f)
   {
-    if (f->geomType() == GEntity::CompoundSurface)return;
+    if (f->geomType() == GEntity::CompoundSurface) return;
+
+    if(CTX::instance()->geom.surfaceType > 0)
+      f->fillVertexArray(f->geomType() == GEntity::ProjectionFace);
+
     Range<double> ubounds = f->parBounds(0);
     Range<double> vbounds = f->parBounds(1);
     const double uav = 0.5 * (ubounds.high() + ubounds.low());
@@ -283,7 +287,7 @@ class drawGFace {
       GPoint p = f->point(uav, vav);
       char Num[100];
       sprintf(Num, "%d", f->tag());
-      double offset = 0.3 * CTX::instance()->glFontSize * _ctx->pixel_equiv_x;
+      double offset = 0.1 * CTX::instance()->glFontSize * _ctx->pixel_equiv_x;
       double x = p.x(), y = p.y(), z = p.z();
       _ctx->transform(x, y, z);
       glRasterPos3d(x + offset / _ctx->s[0],
@@ -307,20 +311,13 @@ class drawGFace {
   }
   void _drawPlaneGFace(GFace *f)
   {
+    if(CTX::instance()->geom.surfaceType > 0)
+      f->fillVertexArray();
+
     if(!CTX::instance()->geom.surfaceType || !f->va_geom_triangles ||
-       CTX::instance()->geom.surfacesNum || CTX::instance()->geom.normals){
-      // We create data here and the routine is not designed to be
-      // reentrant, so we must lock it to avoid race conditions when
-      // redraw events are fired in rapid succession
-      static bool busy = false;
-      if(f->cross.empty() && !busy) {
-        busy = true; 
-        f->buildRepresentationCross();
-        busy = false;
-      }
-    }
+       CTX::instance()->geom.surfacesNum || CTX::instance()->geom.normals)
+      f->buildRepresentationCross();
 
-    //FIXME: cleanup buildGraphicsRep
     if(CTX::instance()->geom.surfaces) {
       if(CTX::instance()->geom.surfaceType > 0 && f->va_geom_triangles){
         _drawVertexArray(f->va_geom_triangles, CTX::instance()->geom.light, 
@@ -347,7 +344,7 @@ class drawGFace {
     if(CTX::instance()->geom.surfacesNum) {
       char Num[100];
       sprintf(Num, "%d", f->tag());
-      double offset = 0.3 * CTX::instance()->glFontSize * _ctx->pixel_equiv_x;
+      double offset = 0.1 * CTX::instance()->glFontSize * _ctx->pixel_equiv_x;
       double x = 0.5 * (f->cross[0].x() + f->cross[1].x());
       double y = 0.5 * (f->cross[0].y() + f->cross[1].y());
       double z = 0.5 * (f->cross[0].z() + f->cross[1].z());
@@ -450,7 +447,7 @@ class drawGRegion {
     if(CTX::instance()->geom.volumesNum){
       char Num[100];
       sprintf(Num, "%d", r->tag());
-      double offset = (0.5 * size + 0.3 * CTX::instance()->glFontSize) *
+      double offset = (0.5 * size + 0.1 * CTX::instance()->glFontSize) *
         _ctx->pixel_equiv_x;
       glRasterPos3d(x + offset / _ctx->s[0],
                     y + offset / _ctx->s[1],
