@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -11,13 +11,15 @@ class GEdge;
 // Create the mesh of the edge
 class meshGEdge {
  public :
-  void operator () ( GEdge * );
+  meshGEdge() {}
+  void operator () (GEdge *);
 };
 
 // destroy the mesh of the edge
 class deMeshGEdge {
  public :
-  void operator () ( GEdge * );
+  deMeshGEdge() {}
+  void operator () (GEdge *);
 };
 
 int MeshExtrudedCurve(GEdge *ge);
