@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,21 +8,24 @@
 
 #include <vector>
 #include <set>
+#include "GFaceCompound.h"
 
 class GFace;
 class MVertex;
+class GFaceCompound;
 
 // Create the mesh of the face
 class meshGFace {
   const bool repairSelfIntersecting1dMesh;
  public :
-  meshGFace (bool r = true) : repairSelfIntersecting1dMesh(r) {}
+  meshGFace (bool r = true, bool s = false) : repairSelfIntersecting1dMesh(r){}
   void operator () (GFace *);
 };
 
 // Destroy the mesh of the face
 class deMeshGFace {
  public :
+  deMeshGFace (bool s = false){}
   void operator () (GFace *);
 };
 
@@ -42,5 +45,8 @@ void findTransfiniteCorners(GFace *gf, std::vector<MVertex*> &corners);
 int MeshTransfiniteSurface(GFace *gf);
 int MeshExtrudedSurface(GFace *gf, std::set<std::pair<MVertex*, MVertex*> > 
                         *constrainedEdges=0);
+int MeshBoundaryLayerSurface(GFace *gf);
+void partitionAndRemesh(GFaceCompound *gf);
+bool checkMeshCompound(GFaceCompound *gf, std::list<GEdge*> &edges);
 
 #endif
