@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -16,9 +16,9 @@ StringXNumber AnnotateOptions_Number[] = {
   {GMSH_FULLRC, "X", GMSH_AnnotatePlugin::callbackX, 50.},
   {GMSH_FULLRC, "Y", GMSH_AnnotatePlugin::callbackY, 30.},
   {GMSH_FULLRC, "Z", GMSH_AnnotatePlugin::callbackZ, 0.},
-  {GMSH_FULLRC, "ThereD", GMSH_AnnotatePlugin::callback3D, 0.},
+  {GMSH_FULLRC, "ThreeD", GMSH_AnnotatePlugin::callback3D, 0.},
   {GMSH_FULLRC, "FontSize", GMSH_AnnotatePlugin::callbackFontSize, 14.},
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 StringXString AnnotateOptions_String[] = {
@@ -177,18 +177,17 @@ std::string GMSH_AnnotatePlugin::callbackAlign(int num, int action, std::string
 
 std::string GMSH_AnnotatePlugin::getHelp() const
 {
-  return "Plugin(Annotate) adds the text string `Text',\n"
-         "in font `Font' and size `FontSize', in the view\n"
-         "`iView'. If `ThreeD' is equal to 1, the plugin inserts\n"
-         "the string in model coordinates at the position\n"
-         "(`X',`Y',`Z'). If `ThreeD' is equal to 0, the plugin\n"
-         "inserts the string in screen coordinates at\n"
-         "the position (`X',`Y'). The string is aligned\n"
-         "according to `Align'. If `iView' < 0, the plugin\n"
-         "is run on the current view.\n"
-         "\n"
-         "Plugin(Annotate) is executed in-place for list-based\n"
-         "datasets or creates a new view for other datasets.\n";
+  return "Plugin(Annotate) adds the text string `Text', "
+    "in font `Font' and size `FontSize', in the view "
+    "`View'. The string is aligned according to `Align'.\n\n"
+    "If `ThreeD' is equal to 1, the plugin inserts "
+    "the string in model coordinates at the position "
+    "(`X',`Y',`Z'). If `ThreeD' is equal to 0, the plugin "
+    "inserts the string in screen coordinates at "
+    "the position (`X',`Y').\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(Annotate) is executed in-place for list-based "
+    "datasets or creates a new view for other datasets.";
 }
 
 int GMSH_AnnotatePlugin::getNbOptions() const
