@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,7 +8,7 @@
 #include "GmshDefines.h"
 
 StringXNumber CurlOptions_Number[] = {
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 extern "C"
@@ -21,11 +21,10 @@ extern "C"
 
 std::string GMSH_CurlPlugin::getHelp() const
 {
-  return "Plugin(Curl) computes the curl of the field\n"
-         "in the view `iView'. If `iView' < 0, the plugin\n"
-         "is run on the current view.\n"
-         "\n"
-         "Plugin(Curl) creates one new view.\n";
+  return "Plugin(Curl) computes the curl of the field "
+    "in the view `View'.\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(Curl) creates one new view.";
 }
 
 int GMSH_CurlPlugin::getNbOptions() const
@@ -38,21 +37,6 @@ StringXNumber *GMSH_CurlPlugin::getOption(int iopt)
   return &CurlOptions_Number[iopt];
 }
 
-static std::vector<double> *incrementList(PViewDataList *data2, int type)
-{
-  switch(type){
-  case TYPE_PNT: data2->NbVP++; return &data2->VP;
-  case TYPE_LIN: data2->NbVL++; return &data2->VL;
-  case TYPE_TRI: data2->NbVT++; return &data2->VT;
-  case TYPE_QUA: data2->NbVQ++; return &data2->VQ;
-  case TYPE_TET: data2->NbVS++; return &data2->VS;
-  case TYPE_HEX: data2->NbVH++; return &data2->VH;
-  case TYPE_PRI: data2->NbVI++; return &data2->VI;
-  case TYPE_PYR: data2->NbVY++; return &data2->VY;
-  default: return 0;
-  }
-}
-
 PView *GMSH_CurlPlugin::execute(PView *v)
 {
   int iView = (int)CurlOptions_Number[0].def;
@@ -60,7 +44,7 @@ PView *GMSH_CurlPlugin::execute(PView *v)
   PView *v1 = getView(iView, v);
   if(!v1) return v;
 
-  PViewData *data1 = v1->getData();
+  PViewData *data1 = v1->getData(true); // get adaptive data if available
   if(data1->hasMultipleMeshes()){
     Msg::Error("Curl plugin cannot be run on multi-mesh views");
     return v;
@@ -75,7 +59,7 @@ PView *GMSH_CurlPlugin::execute(PView *v)
       int numComp = data1->getNumComponents(0, ent, ele);
       if(numComp != 3) continue;
       int type = data1->getType(0, ent, ele);
-      std::vector<double> *out = incrementList(data2, type);
+      std::vector<double> *out = data2->incrementList(3, type);
       if(!out) continue;
       int numNodes = data1->getNumNodes(0, ent, ele);
       double x[8], y[8], z[8], val[8 * 3];
