@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -26,6 +26,10 @@ class GMSH_CutGridPlugin : public GMSH_PostPlugin
  public:
   GMSH_CutGridPlugin(){}
   std::string getName() const { return "CutGrid"; }
+  std::string getShortHelp() const
+  {
+    return "Cut with a regular grid defined by 3 points";
+  }
   std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
