@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -28,6 +28,10 @@ class GMSH_CutParametricPlugin : public GMSH_PostPlugin
  public:
   GMSH_CutParametricPlugin(){}
   std::string getName() const { return "CutParametric"; }
+  std::string getShortHelp() const
+  {
+    return "Cut with the parametric curve (X(u), Y(u), Z(u))";
+  }
   std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
