@@ -1,16 +1,16 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include "Eigenvectors.h"
 #include "Numeric.h"
-#include "EigSolve.h"
+#include "fullMatrix.h"
 #include "GmshDefines.h"
 
 StringXNumber EigenvectorsOptions_Number[] = {
   {GMSH_FULLRC, "ScaleByEigenvalues", NULL, 1.},
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 extern "C"
@@ -23,17 +23,15 @@ extern "C"
 
 std::string GMSH_EigenvectorsPlugin::getHelp() const
 {
-  return "Plugin(Eigenvectors) computes the three (right)\n"
-         "eigenvectors of each tensor in the view `iView'\n"
-         "and sorts them according to the value of the\n"
-         "associated eigenvalues. If `ScaleByEigenvalues'\n"
-         "is set, each eigenvector is scaled by its\n"
-         "associated eigenvalue. The plugin gives an error\n"
-         "if the eigenvectors are complex. If `iView' < 0,\n"
-         "the plugin is run on the current view.\n"
-         "\n"
-         "Plugin(Eigenvectors) creates three new\n"
-         "vector views.\n";
+  return "Plugin(Eigenvectors) computes the three (right) "
+    "eigenvectors of each tensor in the view `View' "
+    "and sorts them according to the value of the "
+    "associated eigenvalues.\n\n"
+    "If `ScaleByEigenvalues' is set, each eigenvector is "
+    "scaled by its associated eigenvalue. The plugin "
+    "gives an error if the eigenvectors are complex.\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(Eigenvectors) creates three new vector view.";
 }
 
 int GMSH_EigenvectorsPlugin::getNbOptions() const
@@ -46,28 +44,6 @@ StringXNumber *GMSH_EigenvectorsPlugin::getOption(int iopt)
   return &EigenvectorsOptions_Number[iopt];
 }
 
-static std::vector<double> *incrementList(PViewDataList *data2, int type)
-{
-  switch(type){
-  case TYPE_PNT: data2->NbVP++; return &data2->VP;
-  case TYPE_LIN: data2->NbVL++; return &data2->VL;
-  case TYPE_TRI: data2->NbVT++; return &data2->VT;
-  case TYPE_QUA: data2->NbVQ++; return &data2->VQ;
-  case TYPE_TET: data2->NbVS++; return &data2->VS;
-  case TYPE_HEX: data2->NbVH++; return &data2->VH;
-  case TYPE_PRI: data2->NbVI++; return &data2->VI;
-  case TYPE_PYR: data2->NbVY++; return &data2->VY;
-  default: return 0;
-  }
-}
-
-static int nonzero(double v[3])
-{
-  for(int i = 0; i < 3; i++)
-    if(fabs(v[i]) > 1.e-16) return 1;
-  return 0;
-}
-
 PView *GMSH_EigenvectorsPlugin::execute(PView *v)
 {
   int scale = (int)EigenvectorsOptions_Number[0].def;
@@ -91,16 +67,17 @@ PView *GMSH_EigenvectorsPlugin::execute(PView *v)
   PViewDataList *dmax = getDataList(max);
 
   int nbcomplex = 0;
-
+  fullMatrix<double> mat(3, 3), vl(3, 3), vr(3, 3);
+  fullVector<double> dr(3), di(3);
   for(int ent = 0; ent < data1->getNumEntities(0); ent++){
     for(int ele = 0; ele < data1->getNumElements(0, ent); ele++){
       if(data1->skipElement(0, ent, ele)) continue;
       int numComp = data1->getNumComponents(0, ent, ele);
       if(numComp != 9) continue;
       int type = data1->getType(0, ent, ele);
-      std::vector<double> *outmin = incrementList(dmin, type);
-      std::vector<double> *outmid = incrementList(dmid, type);
-      std::vector<double> *outmax = incrementList(dmax, type);
+      std::vector<double> *outmin = dmin->incrementList(3, type);
+      std::vector<double> *outmid = dmid->incrementList(3, type);
+      std::vector<double> *outmax = dmax->incrementList(3, type);
       if(!outmin || !outmid || !outmax) continue;
       int numNodes = data1->getNumNodes(0, ent, ele);
       double xyz[3][8];
@@ -115,21 +92,21 @@ PView *GMSH_EigenvectorsPlugin::execute(PView *v)
       }
       for(int step = 0; step < data1->getNumTimeSteps(); step++){
         for(int nod = 0; nod < numNodes; nod++){
-          double val[9];
-          for(int comp = 0; comp < numComp; comp++)
-            data1->getValue(step, ent, ele, nod, comp, val[comp]);
-          double wr[3], wi[3], B[9];
-          if(!EigSolve3x3(val, wr, wi, B))
-            Msg::Error("Eigensolver failed to converge");
-          nbcomplex += nonzero(wi); 
-          if(!scale) wr[0] = wr[1] = wr[2] = 1.;
-          for(int i = 0; i < 3; i++){
-            double res;
-            // wrong if there are complex eigenvals (B contains both
-            // real and imag parts: cf. explanation in EigSolve.cpp)
-            res = wr[0] * B[i]; outmin->push_back(res);
-            res = wr[1] * B[3 + i]; outmid->push_back(res);
-            res = wr[2] * B[6 + i]; outmax->push_back(res);
+          for(int i = 0; i < 3; i++)
+            for(int j = 0; j < 3; j++)
+              data1->getValue(step, ent, ele, nod, 3 * i + j, mat(i, j));
+          if(mat.eig(dr, di, vl, vr, true)){
+            if(!scale) dr(0) = dr(1) = dr(2) = 1.;
+            for(int i = 0; i < 3; i++){
+              double res;
+              res = dr(0) * vr(i, 0); outmin->push_back(res);
+              res = dr(1) * vr(i, 1); outmid->push_back(res);
+              res = dr(2) * vr(i, 2); outmax->push_back(res);
+            }
+            if(di(0) || di(1) || di(2)) nbcomplex++;
+          }
+          else{
+            Msg::Error("Could not compute eigenvalues/vectors");
           }
         }
       }
@@ -137,8 +114,7 @@ PView *GMSH_EigenvectorsPlugin::execute(PView *v)
   }
 
   if(nbcomplex)
-    Msg::Error("%d tensors have complex eigenvalues/eigenvectors", 
-               nbcomplex);
+    Msg::Error("%d tensors have complex eigenvalues", nbcomplex);
   
   for(int i = 0; i < data1->getNumTimeSteps(); i++){
     double time = data1->getTime(i);
