@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -18,6 +18,10 @@ class GMSH_Lambda2Plugin : public GMSH_PostPlugin
  public:
   GMSH_Lambda2Plugin(){}
   std::string getName() const { return "Lambda2"; }
+  std::string getShortHelp() const
+  {
+    return "Detect vortices";
+  }
   std::string getHelp() const;
   std::string getAuthor() const { return "E. Marchandise"; }
   int getNbOptions() const;
