@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -12,14 +12,15 @@ class GMSH_LevelsetPlugin : public GMSH_PostPlugin
 {
  private:
   double _invert;
-  void _addElement(int step, int np, int numEdges, int numComp,
+  void _addElement(int np, int numEdges, int numComp,
                    double xp[12], double yp[12], double zp[12],
-                   double valp[12][9], PViewDataList *out);
+                   double valp[12][9], PViewDataList *out,
+                   bool firstStep);
   void _cutAndAddElements(PViewData *vdata, PViewData *wdata,
                           int ent, int ele, int step, int wstep,
                           double x[8], double y[8], double z[8],
                           double levels[8], double scalarValues[8],
-                          PViewDataList *out);
+                          PViewDataList *out, bool firstStep);
  protected:
   double _ref[3], _targetError;
   int _valueTimeStep, _valueView, _valueIndependent, _recurLevel, _extractVolume;
