@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -17,6 +17,10 @@ class GMSH_LongituteLatitudePlugin : public GMSH_PostPlugin
 {
  public:
   std::string getName() const { return "LongitudeLatitude"; }
+  std::string getShortHelp() const
+  {
+    return "Project in longitude-latitude";
+  }
   std::string getHelp() const;
   std::string getAuthor() const { return "J. Lambrechts"; }
   int getNbOptions() const;
