@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,7 +8,7 @@
 StringXNumber ModulusPhaseOptions_Number[] = {
   {GMSH_FULLRC, "RealPart", NULL, 0.},
   {GMSH_FULLRC, "ImaginaryPart", NULL, 1.},
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 extern "C"
@@ -21,14 +21,13 @@ extern "C"
 
 std::string GMSH_ModulusPhasePlugin::getHelp() const
 {
-  return "Plugin(ModulusPhase) interprets the time steps\n"
-         "`realPart' and `imaginaryPart' in the view `iView'\n"
-         "as the real and imaginary parts of a complex field\n"
-         "and replaces them with their corresponding\n"
-         "modulus and phase. If `iView' < 0, the plugin is\n"
-         "run on the current view.\n"
-         "\n"
-         "Plugin(ModulusPhase) is executed in-place.\n";
+  return "Plugin(ModulusPhase) interprets the time steps "
+    "`realPart' and `imaginaryPart' in the view `View' "
+    "as the real and imaginary parts of a complex field "
+    "and replaces them with their corresponding "
+    "modulus and phase.\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(ModulusPhase) is executed in-place.";
 }
 
 int GMSH_ModulusPhasePlugin::getNbOptions() const
