@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -21,6 +21,10 @@ class GMSH_ProbePlugin : public GMSH_PostPlugin
  public:
   GMSH_ProbePlugin(){}
   std::string getName() const { return "Probe"; }
+  std::string getShortHelp() const
+  {
+    return "Get value at point (X, Y, Z)";
+  }
   std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
