@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -9,7 +9,8 @@
 #include "GmshDefines.h"
 
 StringXNumber SkinOptions_Number[] = {
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "Visible", NULL, 1.},
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 extern "C"
@@ -22,11 +23,11 @@ extern "C"
 
 std::string GMSH_SkinPlugin::getHelp() const
 {
-  return "Plugin(Skin) extracts the skin (the boundary) of\n"
-         "the view `iView'. If `iView' < 0, the plugin is run\n"
-         "on the current view.\n"
-         "\n"
-         "Plugin(Skin) creates one new view.\n";
+  return "Plugin(Skin) extracts the boundary (skin) of "
+    "the view `View'. If `Visible' is set, the plugin only "
+    "extracts the skin of visible entities.\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(Skin) creates one new view.";
 }
 
 int GMSH_SkinPlugin::getNbOptions() const
@@ -137,11 +138,12 @@ static int getBoundary(int type, const int (**boundary)[6][4])
 
 PView *GMSH_SkinPlugin::execute(PView *v)
 {
-  int iView = (int)SkinOptions_Number[0].def;
+  int visible = (int)SkinOptions_Number[0].def;
+  int iView = (int)SkinOptions_Number[1].def;
 
   PView *v1 = getView(iView, v);
   if(!v1) return v;
-  PViewData *data1 = v1->getData();
+  PViewData *data1 = v1->getData(true); // get adaptive data if available
 
   if(data1->hasMultipleMeshes()){
     Msg::Error("Skin plugin cannot be applied to multi-mesh views");
@@ -154,11 +156,20 @@ PView *GMSH_SkinPlugin::execute(PView *v)
   std::set<ElmData, ElmDataLessThan> skin;
   ElmDataLessThan::tolerance = CTX::instance()->lc * 1.e-12;
 
-  for(int ent = 0; ent < data1->getNumEntities(0); ent++){
-    for(int ele = 0; ele < data1->getNumElements(0, ent); ele++){
-      if(data1->skipElement(0, ent, ele)) continue;
-      int numComp = data1->getNumComponents(0, ent, ele);
-      int type = data1->getType(0, ent, ele);
+  int firstNonEmptyStep = 0;
+  for(int step = 0; step < data1->getNumTimeSteps(); step++){
+    if(data1->hasTimeStep(step)){
+      firstNonEmptyStep = step;
+      break;
+    }
+  }
+
+  for(int ent = 0; ent < data1->getNumEntities(firstNonEmptyStep); ent++){
+    if(visible && data1->skipEntity(firstNonEmptyStep, ent)) continue;
+    for(int ele = 0; ele < data1->getNumElements(firstNonEmptyStep, ent); ele++){
+      if(data1->skipElement(firstNonEmptyStep, ent, ele, visible)) continue;
+      int numComp = data1->getNumComponents(firstNonEmptyStep, ent, ele);
+      int type = data1->getType(firstNonEmptyStep, ent, ele);
       const int (*boundary)[6][4];
       int numBoundary = getBoundary(type, &boundary);
       if(!numBoundary) continue;
@@ -168,7 +179,7 @@ PView *GMSH_SkinPlugin::execute(PView *v)
           int nod = (*boundary)[i][j];
           if(nod < 0) continue;
           double x, y, z;
-          data1->getNode(0, ent, ele, nod, x, y, z);
+          data1->getNode(firstNonEmptyStep, ent, ele, nod, x, y, z);
           e.x.push_back(x);
           e.y.push_back(y);
           e.z.push_back(z);
@@ -176,13 +187,15 @@ PView *GMSH_SkinPlugin::execute(PView *v)
         std::set<ElmData, ElmDataLessThan>::iterator it = skin.find(e);
         if(it == skin.end()){
           for(int step = 0; step < data1->getNumTimeSteps(); step++){
-            for(int j = 0; j < 4; j++){
-              int nod = (*boundary)[i][j];
-              if(nod < 0) continue;
-              double v;
-              for(int comp = 0; comp < numComp; comp++){
-                data1->getValue(step, ent, ele, nod, comp, v);
-                e.v.push_back(v);
+            if(data1->hasTimeStep(step)){
+              for(int j = 0; j < 4; j++){
+                int nod = (*boundary)[i][j];
+                if(nod < 0) continue;
+                double v;
+                for(int comp = 0; comp < numComp; comp++){
+                  data1->getValue(step, ent, ele, nod, comp, v);
+                  e.v.push_back(v);
+                }
               }
             }
           }
@@ -199,7 +212,8 @@ PView *GMSH_SkinPlugin::execute(PView *v)
     it->addInView(data2);
   
   for(int i = 0; i < data1->getNumTimeSteps(); i++)
-    data2->Time.push_back(data1->getTime(i));
+    if(data1->hasTimeStep(i))
+      data2->Time.push_back(data1->getTime(i));
   data2->setName(data1->getName() + "_Skin");
   data2->setFileName(data1->getName() + "_Skin.pos");
   data2->finalize();
