@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -20,6 +20,10 @@ class GMSH_StreamLinesPlugin : public GMSH_PostPlugin
  public:
   GMSH_StreamLinesPlugin(){}
   std::string getName() const { return "StreamLines"; }
+  std::string getShortHelp() const
+  {
+    return "Compute particle trajectories in velocity field";
+  }
   std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
