@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -19,7 +19,7 @@ StringXNumber TransformOptions_Number[] = {
   {GMSH_FULLRC, "Ty", NULL, 0.}, // cannot use T2 (reserved token in parser)
   {GMSH_FULLRC, "Tz", NULL, 0.}, // cannot use T3 (reserved token in parser)
   {GMSH_FULLRC, "SwapOrientation", NULL, 0.},
-  {GMSH_FULLRC, "iView", NULL, -1.}
+  {GMSH_FULLRC, "View", NULL, -1.}
 };
 
 extern "C"
@@ -32,17 +32,16 @@ extern "C"
 
 std::string GMSH_TransformPlugin::getHelp() const
 {
-  return "Plugin(Transform) transforms the homogeneous\n"
-         "node coordinates (x,y,z,1) of the elements in\n"
-         "the view `iView' by the matrix\n"
-         "[`A11' `A12' `A13' `Tx']\n"
-         "[`A21' `A22' `A23' `Ty']\n"
-         "[`A31' `A32' `A33' `Tz'].\n"
-         "If `SwapOrientation' is set, the orientation of the\n"
-         "elements is reversed. If `iView' < 0, the plugin\n"
-         "is run on the current view.\n"
-         "\n"
-         "Plugin(Transform) is executed in-place.\n";
+  return "Plugin(Transform) transforms the homogeneous "
+    "node coordinates (x,y,z,1) of the elements in "
+    "the view `View' by the matrix\n\n"
+    "[`A11' `A12' `A13' `Tx']\n"
+    "[`A21' `A22' `A23' `Ty']\n"
+    "[`A31' `A32' `A33' `Tz'].\n\n"
+    "If `SwapOrientation' is set, the orientation of the "
+    "elements is reversed.\n\n"
+    "If `View' < 0, the plugin is run on the current view.\n\n"
+    "Plugin(Transform) is executed in-place.";
 }
 
 int GMSH_TransformPlugin::getNbOptions() const
