@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -12,45 +12,48 @@
 #include "StringUtils.h"
 
 bool PViewDataGModel::addData(GModel *model, std::map<int, std::vector<double> > &data,
-                              int step, double time, int partition)
+                              int step, double time, int partition, int numComp)
 {
   if(data.empty()) return false;
 
-  int numComp = 9;
-  for(std::map<int, std::vector<double> >::iterator it = data.begin(); 
-      it != data.end(); it++)
-    numComp = std::min(numComp, (int)it->second.size());
+  if (numComp < 0){
+    numComp = 9;
+    for(std::map<int, std::vector<double> >::iterator it = data.begin();
+        it != data.end(); it++)
+      numComp = std::min(numComp, (int)it->second.size());
+  }
 
   while(step >= (int)_steps.size())
     _steps.push_back(new stepData<double>(model, numComp));
-  
+
   _steps[step]->setTime(time);
-  
-  int numEnt = (_type == NodeData) ? model->getNumMeshVertices() : 
+
+  int numEnt = (_type == NodeData) ? model->getNumMeshVertices() :
     model->getNumMeshElements();
   _steps[step]->resizeData(numEnt);
-  
-  for(std::map<int, std::vector<double> >::iterator it = data.begin(); 
+
+  for(std::map<int, std::vector<double> >::iterator it = data.begin();
       it != data.end(); it++){
-    double *d  = _steps[step]->getData(it->first, true);
-    for(int j = 0; j < numComp; j++)
+    int mult = it->second.size() / numComp;
+    double *d  = _steps[step]->getData(it->first, true, mult);
+    for(int j = 0; j < numComp * mult; j++)
       d[j] = it->second[j];
   }
-  _partitions.insert(partition);
+  _steps[step]->getPartitions().insert(partition);
   finalize();
   return true;
 }
 
 bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
-                              bool binary, bool swap, int step, double time, 
+                              bool binary, bool swap, int step, double time,
                               int partition, int numComp, int numEnt)
 {
-  Msg::Info("Reading step %d (time %g) partition %d: %d records", 
-      step, time, partition, numEnt);
+  Msg::Info("Reading step %d (time %g) partition %d: %d records",
+            step, time, partition, numEnt);
 
   while(step >= (int)_steps.size())
     _steps.push_back(new stepData<double>(GModel::current(), numComp));
-  
+
   _steps[step]->setFileName(fileName);
   _steps[step]->setFileIndex(fileIndex);
   _steps[step]->setTime(time);
@@ -85,7 +88,7 @@ bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
     }
     double *d = _steps[step]->getData(num, true, mult);
     if(binary){
-      if((int)fread(d, sizeof(double), numComp * mult, fp) != numComp * mult) 
+      if((int)fread(d, sizeof(double), numComp * mult, fp) != numComp * mult)
         return false;
       if(swap) SwapBytes((char*)d, sizeof(double), numComp * mult);
     }
@@ -93,13 +96,25 @@ bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
       for(int j = 0; j < numComp * mult; j++)
         if(fscanf(fp, "%lf", &d[j]) != 1) return false;
     }
-    if(numEnt > 100000) 
+    // compute min/max here to avoid calling finalize(true) later:
+    // this would be very slow for large multi-step, multi-partition
+    // datasets (since we would recompute the min/max for all the
+    // previously loaded steps/partitions, and thus loop over all the
+    // elements many times)
+    for(int j = 0; j < mult; j++){
+      double val = ComputeScalarRep(numComp, &d[numComp * j]);
+      _steps[step]->setMin(std::min(_steps[step]->getMin(), val));
+      _steps[step]->setMax(std::max(_steps[step]->getMax(), val));
+      _min = std::min(_min, val);
+      _max = std::max(_max, val);
+    }
+    if(numEnt > 100000)
       Msg::ProgressMeter(i + 1, numEnt, "Reading data");
   }
 
-  _partitions.insert(partition);
+  _steps[step]->getPartitions().insert(partition);
 
-  finalize();
+  finalize(false);
   return true;
 }
 
@@ -114,13 +129,31 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary)
 
   GModel *model = _steps[0]->getModel();
 
-  if(!model->writeMSH(fileName, 2.0, binary, true)) return false;
-
-  // append data
-  FILE *fp = fopen(fileName.c_str(), binary ? "ab" : "a");
-  if(!fp){
-    Msg::Error("Unable to open file '%s'", fileName.c_str());
-    return false;
+  bool writeNodesAndElements = false;
+  FILE *fp;
+  if(writeNodesAndElements){
+    if(!model->writeMSH(fileName, 2.0, binary)) return false;
+    // append data
+    fp = fopen(fileName.c_str(), binary ? "ab" : "a");
+    if(!fp){
+      Msg::Error("Unable to open file '%s'", fileName.c_str());
+      return false;
+    }
+  }
+  else{
+    fp = fopen(fileName.c_str(), binary ? "wb" : "w");
+    if(!fp){
+      Msg::Error("Unable to open file '%s'", fileName.c_str());
+      return false;
+    }
+    fprintf(fp, "$MeshFormat\n");
+    fprintf(fp, "%g %d %d\n", 2.2, binary ? 1 : 0, (int)sizeof(double));
+    if(binary){
+      int one = 1;
+      fwrite(&one, sizeof(int), 1, fp);
+      fprintf(fp, "\n");
+    }
+    fprintf(fp, "$EndMeshFormat\n");
   }
 
   for(unsigned int step = 0; step < _steps.size(); step++){
@@ -171,10 +204,8 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary)
               Msg::Error("Unknown element %d in data", i);
               return false;
             }
-            int mult = 1;
-            if(_type == ElementNodeData)
-              mult = e->getNumVertices();
-            int num = e->getNum();
+            int mult = _steps[step]->getMult(i);
+            int num = model->getMeshElementIndex(e);
             if(binary){
               fwrite(&num, sizeof(int), 1, fp);
               if(_type == ElementNodeData)
@@ -199,7 +230,7 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary)
       }
     }
   }
-    
+
   fclose(fp);
   return true;
 }
@@ -220,7 +251,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
     Msg::Error("Unable to open file '%s'", fileName.c_str());
     return false;
   }
-  
+
   med_int numComp = MEDnChamp(fid, fileIndex + 1);
   if(numComp <= 0){
     Msg::Error("Could not get number of components for MED field");
@@ -231,7 +262,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
   std::vector<char> compName(numComp * MED_TAILLE_PNOM + 1);
   std::vector<char> compUnit(numComp * MED_TAILLE_PNOM + 1);
   med_type_champ type;
-  if(MEDchampInfo(fid, fileIndex + 1, name, &type, &compName[0], &compUnit[0], 
+  if(MEDchampInfo(fid, fileIndex + 1, name, &type, &compName[0], &compUnit[0],
                   numComp) < 0){
     Msg::Error("Could not get MED field info");
     return false;
@@ -240,7 +271,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
   Msg::Info("Reading %d-component field <<%s>>", numComp, name);
   setName(name);
 
-  int numCompMsh = 
+  int numCompMsh =
     (numComp <= 1) ? 1 : (numComp <= 3) ? 3 : (numComp <= 9) ? 9 : numComp;
 
   if(numCompMsh > 9) Msg::Warning("More than 9 components in field");
@@ -249,13 +280,13 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
   // important: it should match the ordering of the MSH element types
   // (when elements are saved without tags, this governs the order
   // with which we implicitly index them in GModel::readMED)
-  const med_entite_maillage entType[] = 
+  const med_entite_maillage entType[] =
     {MED_NOEUD, MED_MAILLE, MED_NOEUD_MAILLE};
-  const med_geometrie_element eleType[] = 
-    {MED_NONE, MED_SEG2, MED_TRIA3, MED_QUAD4, MED_TETRA4, MED_HEXA8, 
-     MED_PENTA6, MED_PYRA5, MED_SEG3, MED_TRIA6, MED_TETRA10, 
+  const med_geometrie_element eleType[] =
+    {MED_NONE, MED_SEG2, MED_TRIA3, MED_QUAD4, MED_TETRA4, MED_HEXA8,
+     MED_PENTA6, MED_PYRA5, MED_SEG3, MED_TRIA6, MED_TETRA10,
      MED_POINT1, MED_QUAD8, MED_HEXA20, MED_PENTA15, MED_PYRA13};
-  const int nodesPerEle[] = 
+  const int nodesPerEle[] =
     {0, 2, 3, 4, 4, 8, 6, 5, 3, 6, 10, 1, 8, 20, 15, 13};
 
   med_int numSteps = 0;
@@ -269,7 +300,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
       }
       if(!i && !j) break; // MED_NOEUD does not care about eleType
     }
-  }    
+  }
 
   if(numSteps < 1 || pairs.empty()){
     Msg::Error("Nothing to import from MED file");
@@ -277,7 +308,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
   }
   else{
     med_entite_maillage ent = entType[pairs[0].first];
-    _type = (ent == MED_NOEUD) ? NodeData : (ent == MED_MAILLE) ? ElementData : 
+    _type = (ent == MED_NOEUD) ? NodeData : (ent == MED_MAILLE) ? ElementData :
       ElementNodeData;
   }
 
@@ -330,7 +361,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
       std::vector<double> val(numVal * numComp);
       char locname[MED_TAILLE_NOM + 1], profileName[MED_TAILLE_NOM + 1];
       if(MEDchampLire(fid, meshName, name, (unsigned char*)&val[0], MED_FULL_INTERLACE,
-                      MED_ALL, locname, profileName, MED_COMPACT, ent, ele, 
+                      MED_ALL, locname, profileName, MED_COMPACT, ent, ele,
                       numdt, numo) < 0){
         Msg::Error("Could not read field values");
         return false;
@@ -357,7 +388,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
           // internal reference element
           for(int i = 0; i < (int)gscoo.size(); i++){
             p.push_back(gscoo[i]);
-            if(i % dim == dim - 1) for(int j = 0; j < 3 - dim; j++) p.push_back(0.); 
+            if(i % dim == dim - 1) for(int j = 0; j < 3 - dim; j++) p.push_back(0.);
           }
         }
       }
@@ -382,9 +413,9 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
 
       // get size of full array and tags (if any) of entities
       bool nodal = (ent == MED_NOEUD);
-      med_int numEnt = MEDnEntMaa(fid, meshName, nodal ? MED_COOR : MED_CONN, 
-                                  nodal ? MED_NOEUD : MED_MAILLE, 
-                                  nodal ? MED_NONE : ele, 
+      med_int numEnt = MEDnEntMaa(fid, meshName, nodal ? MED_COOR : MED_CONN,
+                                  nodal ? MED_NOEUD : MED_MAILLE,
+                                  nodal ? MED_NONE : ele,
                                   nodal ? (med_connectivite)0 : MED_NOD);
       std::vector<med_int> tags(numEnt);
       if(MEDnumLire(fid, meshName, &tags[0], numEnt, nodal ? MED_NOEUD : MED_MAILLE,
@@ -496,7 +527,7 @@ bool PViewDataGModel::writeMED(std::string fileName)
     return false;
   }
 
-  med_int numNodes = MEDnEntMaa(fid, meshName, MED_COOR, MED_NOEUD, 
+  med_int numNodes = MEDnEntMaa(fid, meshName, MED_COOR, MED_NOEUD,
                                 MED_NONE, (med_connectivite)0);
   if(numNodes <= 0){
     Msg::Error("Could not get valid number of nodes in mesh");
@@ -515,7 +546,7 @@ bool PViewDataGModel::writeMED(std::string fileName)
     for(unsigned int i = 0; i < profile.size(); i++)
       for(int k = 0; k < numComp; k++)
         val[i * numComp + k] = _steps[step]->getData(indices[i])[k];
-    if(MEDchampEcr(fid, meshName, fieldName, (unsigned char*)&val[0], 
+    if(MEDchampEcr(fid, meshName, fieldName, (unsigned char*)&val[0],
                    MED_FULL_INTERLACE, numNodes, (char*)MED_NOGAUSS, MED_ALL,
                    profileName, MED_COMPACT, MED_NOEUD, MED_NONE, (med_int)step,
                    (char*)"unknown", time, MED_NONOR) < 0) {
@@ -523,7 +554,7 @@ bool PViewDataGModel::writeMED(std::string fileName)
       return false;
     }
   }
-  
+
   if(MEDfermer(fid) < 0){
     Msg::Error("Unable to close file '%s'", (char*)fileName.c_str());
     return false;
@@ -535,7 +566,7 @@ bool PViewDataGModel::writeMED(std::string fileName)
 
 bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
 {
-  Msg::Error("Gmsh must be compiled with MED support to read '%s'", 
+  Msg::Error("Gmsh must be compiled with MED support to read '%s'",
              fileName.c_str());
   return false;
 }
