@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+ * Copyright (C) 1997-2010 C. Geuzaine, J.-F. Remacle
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -35,9 +35,10 @@
 #define GMSH_CLIENT_WARNING      11
 #define GMSH_CLIENT_ERROR        12
 #define GMSH_CLIENT_PROGRESS     13
-#define GMSH_CLIENT_VIEW         20 /* deprecated: use MERGE_FILE instead */
 #define GMSH_CLIENT_MERGE_FILE   20
 #define GMSH_CLIENT_PARSE_STRING 21
+#define GMSH_CLIENT_VERTEX_ARRAY 22
+#define GMSH_CLIENT_SPEED_TEST   30
 #define GMSH_CLIENT_OPTION       100
 #define GMSH_CLIENT_OPTION_1     (GMSH_CLIENT_OPTION+0)
 #define GMSH_CLIENT_OPTION_2     (GMSH_CLIENT_OPTION+1)
@@ -45,8 +46,8 @@
 #define GMSH_CLIENT_OPTION_4     (GMSH_CLIENT_OPTION+3)
 #define GMSH_CLIENT_OPTION_5     (GMSH_CLIENT_OPTION+4)
 
-int  Gmsh_Connect(char *sockname);
-void Gmsh_SendString(int socket, int type, char str[]);
+int Gmsh_Connect(const char *sockname);
+void Gmsh_SendString(int socket, int type, const char *str);
 void Gmsh_Disconnect(int sock);
 
 #endif
